# Boots Client
# www.launchpad.net/boots
# www.launchpad.net/drizzle
# boots/app/client_config.py
#
# ##### BEGIN LICENSE BLOCK #####
# Version: MPL 1.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
#
# The Original Code is Boots Client code.
#
# The Initial Developer of the Original Code is Clark Boylan, Ken
# Brotherton, Max Goodman, Victoria Lewis, David Rosenbaum, and Andreas
# Turriff. Portions created by Clark Boylan, Ken Brotherton, Max Goodman,
# Victoria Lewis, David Rosenbaum, and Andreas Turriff are Copyright (C)
# 2009. All Rights Reserved.
#
# Contributor(s):
#
# ##### END LICENSE BLOCK #####

import optparse
import os
import info

class ClientConfig(object):
    """Class used to read client configuration from files and the cli."""

    def __init__(self, conf_file_loc = None):
        """Initialize a configuration. Optionally takes a path to the rc file.

        This method performs no IO and after initializaion you must call load
        to populate a config."""

        # Specify default values for a client configuration.
        self._defaults = {"command":            None,
                          "database":           None,
                          "script":             None,
                          'lingo':              'sql',
                          "rcfile":             None,
                          "host":               "localhost",
                          "port":               9306,
                          "username":           os.getenv("USER"),
                          "password":           False,
                          "prompt1":            "> ",
                          "prompt2":            "+ ",
                          "terminating_char":   ";",
                          "history_length":     100,
                          "history_file":       os.path.expanduser("~/.boots_history")}
        self._dict = self._defaults.copy()

        # Use default rc location in info if no argument is passed in.
        if conf_file_loc == None:
            conf_file_loc = info.RCFILE
        self.conf_file = conf_file_loc

        # Build cli option parser.
        version_string = "{name} version {version}".format(name = info.NAME, version = info.VERSION)
        self._cli_parser = optparse.OptionParser(version = version_string);
        self._cli_parser.add_option("-c", "--command",
                                    action = "store",
                                    type = "string",
                                    dest = "command",
                                    help = "Execute command and exit")
        self._cli_parser.add_option("-D", "--database",
                                    action = "store",
                                    type = "string",
                                    dest = "database",
                                    help = "Use database")
        self._cli_parser.add_option("-f", "--script",
                                    action = "store",
                                    type = "string",
                                    dest = "script",
                                    help = "Execute commands from file and exit")
        self._cli_parser.add_option('-l', '--lingo',
                                    action = 'store',
                                    type = 'string',
                                    dest = 'lingo',
                                    help = 'Set the lingo to use')
        self._cli_parser.add_option("-X", "--norc",
                                    action = "store_true",
                                    dest = "norc",
                                    help = "Do not read user configuration file")
        self._cli_parser.add_option("-r", "--rc",
                                    action = "store",
                                    type = "string",
                                    dest = "rcfile",
                                    help = "Filename of user configuration file")
        self._cli_parser.add_option("-H", "--host",
                                    action = "store",
                                    type = "string",
                                    dest = "host",
                                    help = "Connect to host")
        self._cli_parser.add_option("-p", "--port",
                                    action = "store",
                                    type = "int",
                                    dest = "port",
                                    help = "Use port number")
        self._cli_parser.add_option("-u", "--user",
                                    action = "store",
                                    type = "string",
                                    dest = "username",
                                    help = "Login with username")
        self._cli_parser.add_option("-P", "--password",
                                    action = "store",
                                    type = "string",
                                    dest = "password",
                                    help = "Connect using password. If none is given, query for password")
        self._cli_parser.add_option("-t", "--terminatingchar",
                                    action = "store",
                                    type = "string",
                                    dest = "terminating_char",
                                    help = "Specify the SQL statement terminating character. Default is ';'.")
        self._cli_parser.add_option("-F", "--historyfile",
                                    action = "store",
                                    type = "string",
                                    dest = "history_file",
                                    help = "Specify file to save history to")
        self._cli_parser.add_option("-L", "--historylength",
                                    action = "store",
                                    type = "int",
                                    dest = "history_length",
                                    help = "Specify max history file length")
 
    def __getitem__(self, key):
        return self._dict[key]

    def __setitem__(self, key, value):
        self._dict[key] = value

    @property
    def options(self):
        """Give entire configuration as a dictionary."""
        return self._dict

    def load(self):
        """Load a configuration from the cli and rc file.

        Options specified in rc file overwrite defaults and cli options
        overwrite rc options."""

        from_file = self.get_file_conf(self.conf_file)
        from_cli = self.get_cli_conf()
        self._dict.update(from_file)
        self._dict.update(from_cli)

    def get_file_conf(self, filepath):
        """Read a configuration from the specified file. Return a dict."""

        file_dict = {}
        try:
            execfile(filepath, {}, file_dict)
        except IOError:
            print("rcfile {0} not found".format(filepath))
        except SyntaxError:
            print("rcfile {0} contains a syntax error".format(filepath))
        except:
            print("unknown error")
        return file_dict

    def get_cli_conf(self):
        """Read a configuration from the cli. Return a dict."""

        (options, args) = self._cli_parser.parse_args()
        tmp_options = vars(options)
        new_options = {}
        for key, value in tmp_options.items():
            if value != None:
                new_options[key] = value
        self._args = args
        return new_options

    def update(self, pairs_to_add):
        """Update configuration with provided dictionary."""
        self._dict.update(pairs_to_add)
