# Boots Client
# www.launchpad.net/boots
# www.launchpad.net/drizzle
# lib/console.py 
#
# ##### BEGIN LICENSE BLOCK #####
# Version: MPL 1.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
#
# The Original Code is Boots Client code.
#
# The Initial Developer of the Original Code is Clark Boylan, Ken
# Brotherton, Max Goodman, Victoria Lewis, David Rosenbaum, and Andreas
# Turriff. Portions created by Clark Boylan, Ken Brotherton, Max Goodman,
# Victoria Lewis, David Rosenbaum, and Andreas Turriff are Copyright (C)
# 2009. All Rights Reserved.
#
# Contributor(s):
#
# ##### END LICENSE BLOCK #####

from boots.api import api
from boots.lib.ui.plain import PlainUI
from boots.lib.ui.generic import ScriptDriver, StringDriver, StdoutPresenter
from boots.lib.ui.components.metacommands import MetaCommandManager, MetaCommands, parse_metacommand
from boots.lib.lingos import lingo
from boots.lib.lingos.lisp import lisp
from boots.lib.lingos import python
from boots.lib.lingos import sql
import sys

# Utility function.
def iterable(obj):
    """Returns True if obj is iterable; otherwise, False is returned."""
    try:
        iter(obj)
        return True
    except TypeError:
        return False

class Console(object):
    def __init__(self, config):
        self.config = config        
        self.metacommandmanager = MetaCommandManager()
        self.metacommands = MetaCommands({
            '\connect':    self.meta_connect,
            '\disconnect': self.meta_disconnect,
            '\quit':       self.quit})
        self.metacommandmanager.add(self.metacommands)

        self.lingos = {}
        for name, lingo_cls in lingo.iter_lingos():
            self.lingos[name] = lingo_cls(self)
        
        self.servers = [api.Server(self.config["host"], self.config["port"],
                                 {"database": self.config["database"]})]
        
        ui = PlainUI(self, self.config["prompt1"], self.config["prompt2"],
                     self.config["history_length"], self.config["history_file"],
                     self.config["lingo"])
        self.metacommandmanager.add(ui.metacommands)
        self.presenter = ui
        if self.config["script"]:
            self.driver = ScriptDriver(self, self.config["script"], self.config["lingo"])
        elif self.config["command"]:
            self.driver = StringDriver(self, self.config["command"], self.config["lingo"])
        else:
            self.driver = ui

    def input_complete(self, command, lingo):
        return self.lingos[lingo].input_complete(command) 
                      
    def run(self, command, lingo="sql"):
        return self.lingos[lingo].execute(command)

    def connect(self, host, port, database):
        server = api.Server(host, port, {"database": database})
        server.connect()
        if server.is_connected:
            self.servers.append(server)
            return True
        else:
            print("Could not connect to", host, port, database)
            return False

    def disconnect(self, host, port):
        found = False
        for server in self.servers:
            if not host or server.host == host and not port or server.port == port:
                server.disconnect()
                self.servers.remove(server)
                found = True
        return found

    def main(self):
        for server in self.servers:
            server.connect()
        
        for command, lingo in self.driver.get_input():
            if not self.metacommands.execute(parse_metacommand(command)):
                result = self.run(command, lingo)
            
                if iterable(result):
                    for row in result:
                        self.presenter.present(row)
                    self.presenter.present("")
                # Don't print None; a return value of None means there was no result.
                elif result != None:
                    self.presenter.present(result)

        self.quit()
        
    def quit(self):
        # Print a trailing newline.
        self.presenter.present('')
        self.driver.unload()
        self.disconnect(None, None)

        sys.exit(0)

    def meta_connect(self, host=None, port=None, database=None):
        if host is None: 
            host = self.config["host"]
            
        if port is None:
            port = self.config["port"]
        else:
            port = int(port)
            
        if database is None:
            database = self.config["database"]

        self.connect(host, port, database)

    def meta_disconnect(self, host=None, port=None):
        if host is not None:
            host = str(host)
        if port is not None:
            port = int(port)

        self.disconnect(host, port)
