# Boots Client
# www.launchpad.net/boots
# www.launchpad.net/drizzle
# lib/lingos/lisp/builtins.py
#
# ##### BEGIN LICENSE BLOCK #####
# Version: MPL 1.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
#
# The Original Code is Boots Client code.
#
# The Initial Developer of the Original Code is Clark Boylan, Ken
# Brotherton, Max Goodman, Victoria Lewis, David Rosenbaum, and Andreas
# Turriff. Portions created by Clark Boylan, Ken Brotherton, Max Goodman,
# Victoria Lewis, David Rosenbaum, and Andreas Turriff are Copyright (C)
# 2009. All Rights Reserved.
#
# Contributor(s):
#
# ##### END LICENSE BLOCK #####
# Defines lisp builtins.

from boots.lib.lingos.lisp import objects
import sys

nil = objects.Null()
t = objects.T()

car_function = objects.car
cdr_function = objects.cdr

def minus_function(*arguments):
    """Returns the difference of the arguments."""
    if len(arguments) == 0:
        return 0
    elif len(arguments) == 1:
        return objects.Integer(-arguments[0].python_integer())
    else:
        return objects.Integer(arguments[0].python_integer() - sum(python_integers(*arguments[1:])))

def divide_function(*arguments):
    """Returns the difference of the arguments."""
    if len(arguments) == 0:
        return 1
    elif len(arguments) == 1:
        return objects.Integer(1 / arguments[0].python_integer())
    else:
        return objects.Integer(arguments[0].python_integer() / reduce(lambda x, y: x * y, python_integers(*arguments[1:]), 1))

def length_function(s):
    """Returns the length of the sequence."""
    if s is nil:
        return 0
    elif isinstance(s, objects.Cons):
        return objects.Integer(len(s.python_list()))
    elif isinstance(s, objects.String):
        return objects.Integer(len(s.python_string()))
    else:
        raise TypeError('Expected a cons, nil or a string')

def elt_function(s, i):
    """Returns the ith element of the sequence s."""
    if isinstance(s, objects.Cons):
        return s.python_list()[i.python_integer()]
    elif isinstance(s, objects.String):
        return s.python_string()[i.python_integer()]
    else:
        raise TypeError('Expected a cons or string')

def gensym_function():
    """Returns a unique symbol that will never be the same as any other symbol even if it has the same name."""
    gensym = objects.Symbol('gensym{0}'.format(_gensym_count), cache = False)
    _gensym_count += 1
    return gensym

def pyeval(code, globals, locals):
    """Evaluate the python code contained in the String code if it is an expression.  Execute it if it is a statement."""
    code = code.python_string()
    
    if code.strip():
        try:
            return eval(code, globals, locals)
            # If eval results in an error, it might be because code is a statement rather than an expression.
        except:
            exec code in globals, locals

def python_integers(*arguments):
    """Returns the arguments converted to python integers."""
    return (argument.python_integer() for argument in arguments)

def initialize(environment, stdin = sys.stdin.readline, stdout = sys.stdout.write):
    """Sets the builtins in environment."""
    environment[objects.Symbol('*lisp-python*')] = {'stdin' : stdin, 'stdout' : stdout, 'python-globals' : {}, 'python-locals' : {}} # Stay out of here; used in the implementation.
    environment[objects.Symbol('car')] = objects.PythonLambda(car_function)
    environment[objects.Symbol('cdr')] = objects.PythonLambda(cdr_function)
    environment[objects.Symbol('cons')] = objects.PythonLambda(lambda x, y: objects.Cons(x, y))
    environment[objects.Symbol('+')] = objects.PythonLambda(lambda *arguments: objects.Integer(sum(python_integers(*arguments))))
    environment[objects.Symbol('-')] = objects.PythonLambda(minus_function)
    environment[objects.Symbol('*')] = objects.PythonLambda(lambda *arguments: objects.Integer(reduce(lambda x, y: x * y, python_integers(*arguments), 1)))
    environment[objects.Symbol('/')] = objects.PythonLambda(divide_function)
    environment[objects.Symbol('print')] = objects.PythonLambda(lambda x: environment[objects.Symbol('*lisp-python*')]['stdout'](str(x) + "\n") or x)
    environment[objects.Symbol('concat')] = objects.PythonLambda(lambda x, y: String(x.python_string() + y.python_string()))
    environment[objects.Symbol('length')] = objects.PythonLambda(length_function)
    environment[objects.Symbol('elt')] = objects.PythonLambda(elt_function)
    environment[objects.Symbol('gensym')] = objects.PythonLambda(gensym_function)
    environment[objects.Symbol('pyeval')] = objects.PythonLambda(lambda code: pyeval(code, environment[objects.Symbol('*lisp-python*')]['python-globals'], environment[objects.Symbol('*lisp-python*')]['python-locals']))
