# Boots Client
# www.launchpad.net/boots
# www.launchpad.net/drizzle
# lib/lingos/lisp/objects.py
#
# ##### BEGIN LICENSE BLOCK #####
# Version: MPL 1.1
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
#
# The Original Code is Boots Client code.
#
# The Initial Developer of the Original Code is Clark Boylan, Ken
# Brotherton, Max Goodman, Victoria Lewis, David Rosenbaum, and Andreas
# Turriff. Portions created by Clark Boylan, Ken Brotherton, Max Goodman,
# Victoria Lewis, David Rosenbaum, and Andreas Turriff are Copyright (C)
# 2009. All Rights Reserved.
#
# Contributor(s):
#
# ##### END LICENSE BLOCK #####
# Defines the lisp object system.

import abc
import copy
import sys

_gensym_count = 1
_symbols = {}

class Object(object):
    """The base class for all lisp objects."""
    __metaclass__ = abc.ABCMeta
    
    def __init__(self):
        pass

    @abc.abstractmethod
    def __str__(self):
        pass

    def evaluate(self, environment):
        """Evaluate the object.  Most objects evaluate to themselves."""
        return self

class Cons(Object):
    """Implements cons."""
    def __init__(self, car, cdr):
        """Creates a cons with car as the car and cdr as the cdr."""
        super(Cons, self).__init__()
        self.car = car
        self.cdr = cdr

    def __str__(self):
        return self.string(True)

    def string(self, parentheses):
        """Returns a string that represents the Cons.  Parentheses are used if parentheses is true."""
        if parentheses:
            left_paren = '('
            right_paren = ')'
        else:
            left_paren = ''
            right_paren = ''
        
        if isinstance(self.car, Cons):
            car_string = self.car.string(True)
        else:
            car_string = str(self.car)

        if self.cdr is Null():
            separator = ''
            cdr_string = ''
        elif isinstance(self.cdr, Cons):
            separator = ' '
            cdr_string = self.cdr.string(False)
        else:
            separator = ' . '
            cdr_string = str(self.cdr)

        return left_paren + car_string + separator + cdr_string + right_paren

    def python_list(self):
        """Converts the list to a python list."""
        if self.cdr is Null():
            return [self.car]
        elif isinstance(self.cdr, Cons):
            return [self.car] + self.cdr.python_list()
        else:
            return [self.car, self.cdr]

    def evaluate(self, environment):
        if self.car is Symbol('if'): # Special case for the if special form.
            condition_expression = car(cdr(self))
            then_expression = car(cdr(cdr(self)))
            else_expression = car(cdr(cdr(cdr(self))))

            if then_expression is Null() or cdr(cdr(cdr(cdr(self)))) is not Null():
                raise SyntaxError('wrong number of arguments for if special form {0}'.format(self))

            if condition_expression.evaluate(environment) != Null():
                return then_expression.evaluate(environment)
            else:
                return else_expression.evaluate(environment)
        elif self.car is Symbol('progn'): # Special case for the progn special form.
            value = Null()

            for argument in self.python_list()[1:]:
                value = argument.evaluate(environment)
            
            return value
        elif self.car is Symbol('setq'): # Special case for the setq special form.
            arguments = self.python_list()
            symbols = arguments[1::2]
            values = arguments[2::2]

            if len(symbols) != len(values) or len(symbols) == 0:
                raise SyntaxError('The number of symbols must equal the number of values and must be non-zero')

            for symbol, value in zip(symbols, values):
                value = value.evaluate(environment)
                environment[symbol] = value

            return value
        elif self.car is Symbol('lambda'): # Special case for the lambda special form.
            argument_symbols = car(cdr(self)).python_list()
            body = Cons(Symbol('progn'), cdr(cdr(self)))
            return Lambda(argument_symbols, body)
        else: # Handle function calls.
            function = self.car.evaluate(environment)
            if not isinstance(function, Lambda):
                raise TypeError('Expected a function')
            else:
                return function.call((argument.evaluate(environment) for argument in self.python_list()[1:]), environment)

def car(l):
    """Returns the first element of the list l or nil if l is nil."""
    if l is Null():
        return Null()
    else:
        return l.car

def cdr(l):
    """Returns the cdr of the list l or nil if l is nil."""
    if l is Null():
        return Null()
    else:
        return l.cdr

class Environment(Object):
    """Implements environments."""
    def __init__(self, symbol_values = {}):
        """Creates a new environment with a single level of scope.  symbol_values is a dictionary mapping Symbols to Objects
        and describes bindings in the current scope."""
        # Too bad Python 2.6 does not have dictionary comprehension like in Python 3.x.
        self._environment = dict(zip([(name, [symbol_values[name]]) for name in symbol_values]))
        self._scopes = [set(symbol_values.keys())]

    def __str__(self):
        return '<environment object>'

    def __len__(self):
        return len(self._environment)

    def __getitem__(self, symbol):
        """Returns the value of symbol in the environment."""
        if not isinstance(symbol, Symbol):
            raise TypeError('Expected a symbol')

        try:
            return self._environment[symbol][-1]
        except KeyError:
            raise KeyError('No binding for {0} exists in the current environment'.format(symbol))

    def __setitem__(self, symbol, value):
        """Sets symbol to value in the environment.  If no binding for symbol exists, a binding is created at the innermost
        scope."""
        if not isinstance(symbol, Symbol):
            raise TypeError('Expected a symbol')

        if symbol not in self._environment:
            self._environment[symbol] = [value]
            self._scopes[-1].add(symbol)
        else:
            self._environment[symbol][-1] = value

    # This function is inefficient if implemented correctly (the code below needs to be fixed) and will be slow anyway.
    # def __delitem__(self, symbol):
    #     """Removes the first binding found for symbol.  Returns True if the symbol is found and False otherwise."""
    #     if not isinstance(symbol, Symbol):
    #         raise TypeError('Expected a symbol')
        
    #     if symbol in self._environment:
    #         self._environment[symbol].pop()

    #         if len(self._environment[symbol]) == 0:
    #             del self._environment[symbol]
            
    #         self._scopes[-1].discard(symbol)
    #         return True

    #     return False

    def push(self, symbol_values):
        """Adds a new innermost lexical scope as determined by symbol_values.  symbol_values is a dictionary mapping Symbols
        to Objects and describes bindings in the current scope."""
        self._scopes.append(set(symbol_values.keys()))

        for symbol in symbol_values:
            if symbol in self._environment:
                self._environment[symbol].append(symbol_values[symbol])
            else:
                self._environment[symbol] = [symbol_values[symbol]]
        
    def pop(self):
        """Removes the innermost scope and returns a dictionary that corresponds to its bindings."""
        if len(self._scopes) <= 1:
            raise RuntimeError('Cannot pop the innermost scope because then no scopes would remain')
        symbols = self._scopes.pop()
        
        for symbol in symbols:
            self._environment[symbol].pop()

            if len(self._environment[symbol]) == 0:
                del self._environment[symbol]
        
        return symbols

class Integer(Object):
    """Implements lisp integers."""
    def __init__(self, python_integer):
        """Returns a new lisp integer."""
        self._python_integer = python_integer

    def __str__(self):
        return str(self._python_integer)

    def python_integer(self):
        return self._python_integer

def add(x, y):
    """Adds the Integers x and y."""
    return x._python_int + y._python_int

class Lambda(Object):
    """Represents a lisp funtion."""
    def __init__(self, argument_symbols, expression):
        """Creates a lisp function with parameters that correspond to argument_symbols.  The body of the function is
        expression which can be either a python callable or a lisp object."""
        self._argument_symbols = argument_symbols
        self._expression = expression

    def __str__(self):
        return '<lambda object>'

    def call(self, arguments, environment):
        """Calls the function with the specified arguments in the specified environent; returns the result of evaluating
        body of the function."""
        if isinstance(self._expression, Object):
            environment.push(dict(zip(self._argument_symbols, arguments)))
            value = self._expression.evaluate(environment)
            environment.pop()
            return value
        else:
            return self._expression(*arguments)

class PythonLambda(Lambda):
    """Represents a lisp function that is implemented in Python."""
    def __init__(self, function):
        """Creates a new lisp function implemented using function."""
        return super(PythonLambda, self).__init__(None, function)
    
    def __str__(self):
        return '<python_lambda object>'

class Quote(Object):
    """Implements quoting.  When evaluated, this object will return the quoted object."""
    def __init__(self, lisp_object):
        """Wraps lisp_object in a quote."""
        super(Quote, self).__init__()
        self._lisp_object = lisp_object

    def __str__(self):
        return "'" + str(self._lisp_object)
    
    def evaluate(self, environment):
        return self._lisp_object

class String(Object):
    """Implements lisp strings."""
    def __init__(self, python_string):
        """Creates a lisp string using python_string."""
        self._python_string = python_string

    def __str__(self):
        return '"' + self._python_string + '"'

    def python_string(self):
        return self._python_string

class Symbol(Object):
    """Implements lisp symbols."""
    def __new__(cls, name, self_evaluating = False, cache = True):
        if cache and name in _symbols:
            return _symbols[name]
        else:
            symbol = super(Symbol, cls).__new__(cls)
            _symbols[name] = symbol
            return symbol
    
    def __init__(self, name, self_evaluating = False):
        super(Symbol, self).__init__()
        self.name = name
        self.self_evaluating = self_evaluating
    
    def __str__(self):
        return self.name

    def evaluate(self, environment):
        if self.self_evaluating:
            return self
        else:
            return environment[self]

class Null(Symbol):
    """The type of nil."""
    def __new__(cls):
        return super(Null, cls).__new__(cls, 'nil', True)

    # arguments is so that this __init__ method can be called by the __new__ method of Symbol.  Otherwise Symbol('nil') will
    # result in errors.
    def __init__(self, *arguments):
        super(Null, self).__init__('nil', True)

    def python_list(self):
        return []

class T(Symbol):
    """The type of t."""
    def __new__(cls):
        return super(T, cls).__new__(cls, 't', True)

    # arguments is so that this __init__ method can be called by the __new__ method of Symbol.  Otherwise Symbol('nil') will
    # result in errors.
    def __init__(self, *arguments):
        super(T, self).__init__('t', True)
