# Utility function
def issubseq(seq, seq2):
    """Determines if the elements the sequence seq form a subsequence of the sequence seq2."""
    if len(seq) == 0:
        return True
    else:
        i = 0
 
        while i < len(seq2) and seq[0] != seq2[i]:
            i += 1
 
        if i == len(seq2):
            return False
 
        return list(seq[1:]) == list(seq2[:i + 1:len(seq)])

 
class Topic(object):
    """A topic in the help system which may contain other topics."""
    def __init__(self, name, title = None, description = None, short_description = None, see_also = None):
        """Creates a new a help topic.  The fields are as described above."""
        self.subtopics = dict()  
        self.title = title
        self.name = name
        self.description = description
        self.short_description = short_description
        self.see_also = see_also
	
    def format(self):
        """Returns a string that represents this topic in human readable form as
        described in detail above."""
        text = "{0.title}\n\n{0.description}\n\nSee also: {0.see_also}".format(self)
        return text
 
    def format_index(self):
        """Returns a string that lists the subtopics of this topic in human
        readable form as described above."""
        lines = list()

        for name, topic in self.subtopics.iteritems():   
            max_name_len = max(len(subtopic.name) for subtopic in self.subtopics.itervalues())
            subtext = "{0:{2}}--{1:30}".format(topic.name, topic.short_description, max_name_len +3)
            lines.append(subtext)
        lines.sort()
        return "\n".join(lines)

    # Queries:
    #
    # A query is used to refer to a topic in the help system.  A query is either
    # a string or a non-empty tuple of strings.  Using a string is equivalent to
    # using a tuple of length one containing that string.  As an example, the
    # tuple ('python', 'syntax', 'csv') refers to the csv subtopic in the syntax
    # topic which is itself a subtopic of the python topic.
 
    def _translate_query(self, query):
        """Converts a query into a tuple.  If query is not a valid query an
        exception is raised."""
        if isinstance(query, str):
            return (query,)
        elif isinstance(query, (tuple, list)):
            assert len(query) != 0
            for item in query:
                assert isinstance(item, str)
            return query
        else:
            raise TypeError('query object {0} is not a string or tuple of strings'.format(query))
 
    def _ensure_key(self, key):
        """Creates a new subtopic that key maps to if none exists."""
        if key not in self.subtopics:
            self.subtopics[key] = Topic(key)
 
    def _walk(self, path = ()):
        """Returns an iterable of all absolute paths."""
        if len(self.subtopics) == 0:
            yield path
        else:
            for subtopic in self.subtopics:
                for path2 in subtopic._walk(path + self.name):
                    yield path2
 
    # The intention is for these methods to be implemented recursively
    # by calling these methods on topics contained in the current
    # topic (ie. this Help object).  Queries are represented as a
    # tuple denoting the topics.  For example, "python syntax csv"
    # would be represented as ('python', 'syntax', 'csv').
 
    def __contains__(self, query):
        """Return True if query refers to a help topic.  OTherwise,
        False is returned."""
        t = self._translate_query(query)
 
        if len(t) == 1:
            return t[0] in self.subtopics
        else:
            return t[0] in self.subtopics and t[1:] in self.subtopics[t[0]]
 
    def __getitem__(self, query):
        """Returns the help topic denoted by query; topics in query
        that do not exist should be created automatically.  query may
        refer to topic with subtopics."""
        t = self._translate_query(query)
        self._ensure_key(t[0])
 
        if len(t) == 1:
            return self.subtopics[t[0]]
        else:
            return self.subtopics[t[0]][t[1:]]
 
    def __setitem__(self, query, value):
        """Sets the help topic denoted by query to value.  topics in
        query that do not exist should be created automatically.
        query may refer to topic with subtopics."""
        t = self._translate_query(query)
 
        if len(t) == 1:
            self.subtopics[t[0]] = value
        else:
            self._ensure_key(t[0])
            self.subtopics[t[0]][t[1:]] = value
 
    def __delitem__(self, query):
        """Deletes the reference to help topic denoted by query.
        query may refer to topic with subtopics."""
        t = self._translate_query(query)
        if len(t) == 1:
            del self.subtopics[t[0]]
        else:
            del self.subtopics[t[0]][t[1:]]
 
    def search(self, query):
        """Returns an iterable of the absolute paths to topics that
        partially match query."""
        t = self._translate_query(query)
        return (path for path in self._walk() if issubseq(t, path))


