import unittest
from boots.api import api
import boots_unit_test

class TestServerConnections(boots_unit_test.BootsBaseTester):
    def test_valid_connect_fqdn(self):
        server = api.Server("capstonedd.cs.pdx.edu", 9306, {})
        server.connect()
        self.assert_(server.is_connected)

    def test_valid_connect_partial1(self):
        server = api.Server("capstonedd.cs", 9306, {})
        server.connect()
        self.assert_(server.is_connected)

    def test_valid_connect_partial2(self):
        server = api.Server("capstonedd", 9306, {})
        server.connect()
        self.assert_(server.is_connected)

    def test_valid_connect_ip(self):
        server = api.Server("131.252.214.178", 9306, {})
        server.connect()
        self.assert_(server.is_connected)

    def test_valid_connect_fqdn_with_db(self):
        server = api.Server("capstonedd.cs.pdx.edu", 9306, {"database": "world"})
        server.connect()
        self.assert_(server.is_connected)

    def test_invalid_connect(self):
        server = api.Server("kororaa.cs.pdx.edu", 9306, {})
        server.connect()
        self.assert_(not server.is_connected)

    def test_valid_disconnect(self):
        server = api.Server("capstonedd.cs.pdx.edu", 9306, {})
        server.connect()
        server.disconnect()
        self.assert_(not server.is_connected)

    def test_invalid_disconnect(self):
        # Todo this should probably be set up to assert an exception.
        server = api.Server("kororaa.cs.pdx.edu", 9306, {})
        server.connect()
        server.disconnect()
        self.assert_(not server.is_connected)

class TestServerExecute(unittest.TestCase):
    def setUp(self):
        self.server = api.Server("capstonedd.cs.pdx.edu", 9306, {"database": "world"})
        self.server.connect()

    def tearDown(self):
        self.server.disconnect()

    def test_query_length_0(self):
        result = list(self.server.execute(" SELECT * FROM countrylanguage WHERE Language = 'foobar';"))
        self.assertEqual(len(result), 0)

    def test_query_length_1(self):
        result = list(self.server.execute("SELECT * FROM country WHERE name = 'Micronesia, Federated States of';"))
        self.assertEqual(len(result), 1)

    def test_query_length_247(self):
        result = list(self.server.execute("SELECT * FROM city WHERE CountryCode = 'USA';"))
        self.assertEqual(len(result), 274)

    def test_query_length_4081(self):
        result = list(self.server.execute("SELECT * FROM city;"))
        self.assertEqual(len(result), 4081)

    def test_query_length_975359(self):
        result = list(self.server.execute("SELECT * FROM city,country;"))
        self.assertEqual(len(result), 975359)

    def test_query_values_country(self):
        result = list(self.server.execute("SELECT * FROM country WHERE name = 'Micronesia, Federated States of';"))
        self.assertEqual(result[0], 
                         {"Code": "FSM",
                          "Name": "Micronesia, Federated States of",
                          "Continent": "Oceania",
                          "Region": "Micronesia",
                          "SurfaceArea": "702.00",
                          "IndepYear": "1990", 
                          "Population": "119000", 
                          "LifeExpectancy": "68.6",
                          "GNP": "212.00",
                          "GNPOld": None,
                          "LocalName": "Micronesia",
                          "GovernmentForm": "Federal Republic",
                          "HeadOfState": "Leo A. Falcam",
                          "Capital": "2689",
                          "Code2": "FM"})

    def test_query_values_city(self):
        result = list(self.server.execute("SELECT * FROM city WHERE Name = 'Palikir';"))
        self.assertEqual(result[0],
                         {"ID": "2689",
                          "Name": "Palikir",
                          "CountryCode": "FSM",
                          "District": "Pohnpei",
                          "Population": "8600"})

    def test_query_identity_same_q(self):
        result1 = list(self.server.execute("SELECT * FROM country;"))
        result2 = list(self.server.execute("SELECT * FROM country;"))
        self.assertEqual(result1, result2)

    def test_query_identity_equiv_q(self):
        result1 = list(self.server.execute("SELECT * FROM city;"))
        result2 = list(self.server.execute("SELECT ID, Name, CountryCode, District, Population FROM city;"))
        self.assertEqual(result1, result2)

if __name__ == "__main__":
    suite = unittest.TestLoader().loadTestsFromTestCase(TestServerConnections)
    unittest.TextTestRunner(verbosity=2).run(suite)
    suite = unittest.TestLoader().loadTestsFromTestCase(TestServerExecute)
    unittest.TextTestRunner(verbosity=2).run(suite)
